﻿BEGIN TRY
	PRINT '>>> indexreorganize.sql Start';
	PRINT SYSDATETIME();

	-- Ensure a USE <databasename> statement has been executed first.
	USE [CTMDB];

	SET NOCOUNT ON;
	DECLARE @schemaname nvarchar(130);
	DECLARE @objectname nvarchar(130);
	DECLARE @indexname nvarchar(130);
	DECLARE @fragment float;
	DECLARE @command nvarchar(4000);

	-- Conditionally select tables and indexes from the sys.dm_db_index_physical_stats function
	-- and convert object and index IDs to names.
	SELECT	QUOTENAME(S.name) AS schemaname,
			QUOTENAME(O.name) AS objectname,
			QUOTENAME(I.name) AS indexname,
			PS.avg_fragmentation_in_percent AS fragment
	INTO	#work_to_do 
	FROM	sys.dm_db_index_physical_stats (DB_ID(), NULL, NULL , NULL, 'LIMITED') AS PS
			INNER JOIN sys.objects AS O ON PS.object_id = O.object_id
			INNER JOIN sys.schemas AS S ON O.schema_id = S.schema_id
			INNER JOIN sys.indexes AS I ON PS.object_id = I.object_id AND PS.index_id = I.index_id
	WHERE	avg_fragmentation_in_percent >= 10.0 AND PS.index_id > 0;

	-- Declare the cursor for the list of partitions to be processed.
	DECLARE cursorIndex CURSOR FOR SELECT * FROM #work_to_do;

	-- Open the cursor.
	OPEN cursorIndex;

	-- Loop through the partitions.
	WHILE (1=1)
		BEGIN;
			FETCH NEXT FROM cursorIndex INTO @schemaname, @objectname, @indexname, @fragment;
			IF @@FETCH_STATUS < 0 BREAK;
			-- Execute reorganizing.
			SET @command = N'ALTER INDEX ' + @indexname + N' ON ' + @schemaname + N'.' + @objectname + N' REORGANIZE';
			EXEC (@command);
			PRINT N'Executed: ' + @command;
		END;

	-- Close and deallocate the cursor.
	CLOSE cursorIndex;
	DEALLOCATE cursorIndex;

	-- Drop the temporary table.
	DROP TABLE #work_to_do;

	PRINT '>>> indexreorganize.sql End';
	PRINT SYSDATETIME();
END TRY

-- 例外処理
BEGIN CATCH
    PRINT ERROR_MESSAGE();
    ROLLBACK TRANSACTION;
	PRINT '>>> indexreorganize.sql Error';
	PRINT SYSDATETIME();
END CATCH;